@@ -19,27 +19,34 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
-import numpy as np
-import gym, random, re, timeit, copy
-import glob, os, argparse, json, ray, torch
-from matplotlib import pyplot as plt
-import dill
+import argparse
+import copy
+import glob
+import json
+import os
+import random
+import re
+import timeit
 from collections import defaultdict
-from smarts.core.utils.episodes import episodes
-from smarts.core.agent import AgentSpec, Agent
+
+import dill
+import gym
+import numpy as np
+import ray
+import torch
+from matplotlib import pyplot as plt
+
+from smarts.core.agent import Agent, AgentSpec
+from smarts.core.agent_interface import AgentInterface, AgentType, NeighborhoodVehicles
 from smarts.core.controllers import ActionSpaceType
-from smarts.core.agent_interface import (
-    AgentInterface,
-    AgentType,
-    NeighborhoodVehicles,
-)
+from smarts.core.utils.episodes import episodes
+from ultra.scenarios.analysis.base_analysis import BaseAnalysis
+from ultra.scenarios.common.social_vehicle_definitions import get_social_vehicle_color
 from ultra.scenarios.common.visualization import (
-    draw_intersection,
     convert_to_gif,
+    draw_intersection,
     profile_vehicles,
 )
-from ultra.scenarios.common.social_vehicle_definitions import get_social_vehicle_color
-from ultra.scenarios.analysis.base_analysis import BaseAnalysis
 
 num_gpus = 1 if torch.cuda.is_available() else 0
 
@@ -429,11 +436,15 @@ class ScenarioAnalysis(BaseAnalysis):
         )
 
         self.save_histogram(
-            num_vehicle_list, f"{save_dir}/num_vehicles.png", "num_vehicles",
+            num_vehicle_list,
+            f"{save_dir}/num_vehicles.png",
+            "num_vehicles",
         )
 
         self.save_histogram(
-            failed_list, f"{save_dir}/failed_vehicles.png", "failed_vehicles",
+            failed_list,
+            f"{save_dir}/failed_vehicles.png",
+            "failed_vehicles",
         )
 
         if total_south_west > 0:
